var redis = require('redis');
var debug = require('debug')('RedisClientWrapper');

var default_port = 6379;
var default_host = "127.0.0.1";
var default_db_num = 0;


function RedisClientWrapper() {

    this.default_exisis = false;

    this.clients = [];
}

RedisClientWrapper.prototype.getClientByName = function (name) {
    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].name === name) {
            return this.clients[i].client;
        }
    }
    return null;
}


RedisClientWrapper.prototype.getClientByDbNum = function (db_num) {
    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].db_num == db_num) {
            return this.clients[i].client;
        }
    }
    return null;
}

RedisClientWrapper.prototype.getClientByHost = function (host) {
    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].host == host) {
            return this.clients[i].client;
        }
    }
    return null;
}

RedisClientWrapper.prototype.getClientByPort = function (port) {
    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].port == port) {
            return this.clients[i].client;
        }
    }
    return null;
}

RedisClientWrapper.prototype.getDefaultClient = function () {
    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].default == true) {
            return this.clients[i].client;
        }
    }
    return null;
}

RedisClientWrapper.prototype.getClient = function () {
    if (this.clients.length == 0) {
        debug("No Redis client created. RedisClientWrapper.createClient(port, 'host', option)");
        return null;
    }

    if (arguments.length > 0) {
        var index = arguments[0];
        if (typeof index == "number") {
            if (typeof this.clients[index] != "undefined") {
                return this.clients[index].client;
            }
            return null;
        }

        if (typeof index == "string") {
            return this.getClientByName(index);
        }
    }

    if (this.default_exisis) {
        return this.getDefaultClient();
    }

    return this.clients[0].client;
}

/**
 * create redis client
 */
RedisClientWrapper.prototype.createClient = function () {
    var host;
    var port;
    var db_num = null;
    var options;

    var client_name = null;
    var is_default = false;

    if (arguments.length == 1 && typeof arguments[0] == "object") {
        host = arguments[0]['host'] || default_host;
        port = arguments[0]['port'] || default_port;
        db_num = arguments[0]['db_num'] || default_db_num;
        options = arguments[0]['options'] || {};
        client_name = arguments[0]['client_name'] || null;
        is_default = arguments[0]['default'] || false;
    } else
    //.createClient(port, host)
    if (arguments.length == 2) {
        port = arguments[0];
        host = arguments[1];

        options = arguments[2];
    } else
    //.createClient(port, host, db_num||options)
    if (arguments.length == 3) {
        port = arguments[0];
        host = arguments[1];

        //.createClient(port, host, options)
        if (typeof arguments[2] == "object") {
            options = arguments[2];
        } else
        //.createClient(port, host, db_num)
        {
            db_num = arguments[2];
        }
    } else

    //.createClient(port, host, db_num, options)
    if (arguments.length == 4) {
        port = arguments[0];
        host = arguments[1];
        db_num = arguments[2];
        options = arguments[3];
    }

    var redisClient = redis.createClient(port, host, options);

    if (db_num) {

        redisClient.select(db_num, function (error, result) {
            if (error) {
                log(error);
            }
            if (result == "OK") {
                debug("db [" + db_num + "] selected");
            }

        });
    }

    var clientObject = {client: redisClient, host: host, port: port, db_num: db_num};

    if (client_name) {
        clientObject.name = client_name;
    }


    clientObject.default = is_default;

    if (is_default) {
        this.default_exisis = true;
    }

    this.clients.push(clientObject);
    return redisClient;
}

/**
 * assign name for redis client
 * @param redisClient
 * @param name
 * @returns {boolean}
 */
RedisClientWrapper.prototype.setClientName = function (redisClient, name) {
    if (typeof redisClient == "undefined" || !(redisClient instanceof redis.RedisClient)) {
        debug("The first parameter must be an instance of RedisClient, type of " + (typeof redisClient) + " given");
        return false;
    }

    if (typeof name == "undefined" || name instanceof String) {
        debug("The second parameter must be an String, type of " + (typeof name) + " given");
        return false;
    }

    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].client === redisClient) {
            this.clients[i].name = name;
            return true;
        }
    }
    debug("No wrapped redis client found.");
    return false;
}

/**
 * set client default
 *
 * @param redisClient
 * @param is_default
 * @returns {boolean}
 */
RedisClientWrapper.prototype.setClientDefault = function (redisClient, is_default) {
    if (typeof redisClient == "undefined" || !(redisClient instanceof redis.RedisClient)) {
        debug("The first parameter must be an instance of RedisClient, type of " + (typeof redisClient) + " given");
        return false;
    }

    if (typeof is_default != "boolean" && typeof is_default != "undefined") {
        debug("The second parameter must be an Boolean, type of " + (typeof is_default) + " given");
        return false;
    }

    if (typeof is_default == "undefined") {
        is_default = true;
    }

    for (var i = 0; i < this.clients.length; i++) {
        if (this.clients[i].client === redisClient) {
            this.clients[i].default = is_default;

            //----------- set default exists

            this.default_exisis = false;
            for (var j = 0; j < this.clients.length; j++) {
                if (this.clients[j].default === true) {
                    this.default_exisis = true;
                    break;
                }
            }

            //-----------
            return true;
        }
    }

    debug("No wrapped redis client found.");
    return false;
}

module.exports = new RedisClientWrapper();